 // ***********************************************************
// Use this code at your own risk.
//
// Direct questions and/or fixes to:
//   Illah Nourbakhsh     illah@cs.cmu.edu
//   Kevin Wilson         kwilson@cs.cmu.edu
//   Carnegie Mellon University Robotics Institute
//
// Last updated:  June 4, 1998 (SCOUT compatible)
//
// ***********************************************************

// interface to robot through serial port

#include <stdlib.h>

#include "interface.h"
#include "interfaceprivate.h"

const int MaxLines = 100;
// maximum number of lines in the output box.

SensorType Sensors; // the global state that the user looks at.

extern int LineAvailable, InputBarrier, LineWait;  // input box synchronization

extern int UpArrow, DownArrow, LeftArrow, RightArrow, SpaceBar, EnterKey,
	ZeroNow, SonarState, FooState, BarState;
// a bunch of user input state that is set in generic.c

HANDLE hCom1;  // serial port handle
int SonarsActive = 0;
int	SonarActive[NUMSONARS];



// open serial port at 9600 baud with other appropriate parameters.
// returns:
// 0 on success
// -1 on failure to open serial port
// -2 on failure to read port state
// -3 on failure to set port state
int OpenSerial()
{
	DCB dcb;	
	HANDLE hCom;
	DWORD dwError;
	BOOL fSuccess;

	hCom = CreateFile("COM1",
		GENERIC_READ | GENERIC_WRITE,
		0,    /* comm devices must be opened w/exclusive-access */
		NULL, /* no security attrs */
		OPEN_EXISTING, /* comm devices must use OPEN_EXISTING */
		0,    /* not overlapped I/O */
		NULL  /* hTemplate must be NULL for comm devices */
		);

	if (hCom == INVALID_HANDLE_VALUE) {
		dwError = GetLastError();

		return (-1);
	}

	/*
	 * Omit the call to SetupComm to use the default queue sizes.
	 * Get the current configuration.
	*/

	fSuccess = GetCommState(hCom, &dcb);

	if (!fSuccess) {
		return (-2);
	}

	/* Fill in the DCB: baud=19200, 8 data bits, no parity, 1 stop bit. */

	dcb.BaudRate = 38400; // changed 6/4/98 illah
	dcb.fBinary = TRUE;
	dcb.fParity = FALSE;
	
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;

	fSuccess = SetCommState(hCom, &dcb);

	if (!fSuccess) {
		return (-3);
	}

	hCom1 = hCom;
	return (0);
}

int CloseSerial() {
	close(hCom1);
	return (0);
}


// issue Zero command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
int Zero()
{
    char buf[50];
	int i = 0;
	int retval;
	
	Append2Packet(buf, &i, 1); // start byte
	Append2Packet(buf, &i, 2); // low byte of packet length
	Append2Packet(buf, &i, 0); // high byte of packet length
	Append2Packet(buf, &i, 0xd); // zero code
	Append2Packet(buf, &i, 16); // checksum
	Append2Packet(buf, &i, 92); // end byte

	retval = RobotCommand(buf, i, 30000);
//	sprintf(buf, "Zero:%d\n", retval);
//	OutputText(buf);

	return retval; 
}

// issue Limp command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int Limp()
{
    char buf[50];
	int i = 0;
	int retval;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 2);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0xa);
	Append2Packet(buf, &i, 13);
	Append2Packet(buf, &i, 92);

	retval = RobotCommand(buf, i, 4000);
//	sprintf(buf, "Limp:%d\n", retval);
//	OutputText(buf);

	return retval; 
}

// issue ConfAc command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int ConfAc(int ta, int sa, int ra)
{
    char buf[50];
	int i = 0;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 8);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x1);
	Append2Packet(buf, &i, LOWBYTESIGNED(ta));
	Append2Packet(buf, &i, SECONDBYTESIGNED(ta));
	Append2Packet(buf, &i, LOWBYTESIGNED(sa));
	Append2Packet(buf, &i, SECONDBYTESIGNED(sa));
	Append2Packet(buf, &i, LOWBYTESIGNED(ra));
	Append2Packet(buf, &i, SECONDBYTESIGNED(ra));
	Append2Packet(buf, &i, (unsigned char) ((1 + 8 + 0 + 0x1 + (unsigned) LOWBYTESIGNED(ta) 
		+ (unsigned) SECONDBYTESIGNED(ta) 
		+ (unsigned) LOWBYTESIGNED(sa) 
		+ (unsigned) SECONDBYTESIGNED(sa) 
		+ (unsigned) LOWBYTESIGNED(ra) 
		+ (unsigned) SECONDBYTESIGNED(ra)) % 256)); 
	Append2Packet(buf, &i, 92);

	return RobotCommand(buf, i, 4000);
}

// issue ConfTimeout command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int ConfTimeout(int timeout)
{
    char buf[50];
	int i = 0;
	
	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 4);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x12);
	Append2Packet(buf, &i, LOWBYTEUNSIGNED(timeout));
	Append2Packet(buf, &i, SECONDBYTEUNSIGNED(timeout));
	Append2Packet(buf, &i, (unsigned char) ((1 + 4 + 0 + 0x12 + LOWBYTEUNSIGNED(timeout) +
			  SECONDBYTEUNSIGNED(timeout)) % 256)); 
	Append2Packet(buf, &i, 92);

	return RobotCommand(buf, i, 4000);
}

// issue GS command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int GS()
{
    char buf[50];
	int i = 0;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 2);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x7);
	Append2Packet(buf, &i, 10);
	Append2Packet(buf, &i, 92);

	return RobotCommand(buf, i, 4000);
}

// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int RMove(int rightspeed, int leftspeed)
{
	return VM(rightspeed, leftspeed, 0);
}

// issue VM command.
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
int VM(int rightvel, int leftvel, int unused)
{
    char buf[50];
	int i = 0;
	int retval;

	if (rightvel > 350)
		rightvel = 350;
	else if (rightvel < -350)
		rightvel = -350;
	if (leftvel > 350)
		leftvel = 350;
	else if (leftvel < -350)
		leftvel = -350;

	Append2Packet(buf, &i, 1);
	Append2Packet(buf, &i, 8);
	Append2Packet(buf, &i, 0);
	Append2Packet(buf, &i, 0x5);
	Append2Packet(buf, &i, LOWBYTESIGNED(rightvel));
	Append2Packet(buf, &i, SECONDBYTESIGNED(rightvel));
	Append2Packet(buf, &i, LOWBYTESIGNED(leftvel));
	Append2Packet(buf, &i, SECONDBYTESIGNED(leftvel));
	Append2Packet(buf, &i, LOWBYTESIGNED(unused));
	Append2Packet(buf, &i, SECONDBYTESIGNED(unused));
	Append2Packet(buf, &i, (char) ((14 /* 1 + 8 + 0x5 */
			  + LOWBYTESIGNED(rightvel) + SECONDBYTESIGNED(rightvel) + 
			    LOWBYTESIGNED(leftvel) + SECONDBYTESIGNED(leftvel)
			  + LOWBYTESIGNED(unused) + SECONDBYTESIGNED(unused)) & 0xff));
	Append2Packet(buf, &i, 92);

	retval = RobotCommand(buf, i, 4000);

	return retval; 
}

// issue ConfSonars command.
//
// rate is the delay between sonar firings.
// orderlist is an array of sonar numbers.
// orderlist must have 16 elements. Put in the ones you
// want to fire, then 255, then fill the rest with zeroes
// 
// returns 0 on success.
// see RobotCommand in interfaceprivate.c for error return values
// and for data packet format.
// also returns -30 for invalid listsize and -31 for invalid sonar numbers
// in orderlist.
int ConfSonars(int rate, int orderlist[])
{
    char buf[50];
    int result, i, j, sum, size=0;

	for (i=0;i<16;i++) {
		if (((orderlist[i] < 0) || (orderlist[i] >= NUMSONARS)) &&
			(orderlist[i] != 255))
			return -31;
	}

	Append2Packet(buf, &size, 1);
	Append2Packet(buf, &size, 19);
	Append2Packet(buf, &size, 0);
	Append2Packet(buf, &size, 0xF);
	Append2Packet(buf, &size, (unsigned char) rate);
	sum = 1 + 19 + 0xF + rate;
	for (i=0; i < 16; i++) {
		Append2Packet(buf, &size, (unsigned char) orderlist[i]);
		sum += orderlist[i];
	}

	Append2Packet(buf, &size, (char) (sum & 0xFF));
	Append2Packet(buf, &size, 92);
	
	if (result = RobotCommand(buf, size, 4000)) {
		return result;
	}

	return 0;
	
}

// turn all sonars on
int SonarsOn()
{
	int orderlist[16] = {0,3,6,9,12,15,2,5,8,11,14,1,4,7,10,13}; 
	int retval;
//	char buf[256];

	retval = ConfSonars(2, orderlist);		
//	sprintf(buf, "SonarsOn:%d\n", retval);
//	OutputText(buf);
	return retval;
}

// turn all sonars off
int SonarsOff()
{
	int orderlist[16] = {255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; 
	int retval;
//	char buf[256];
	
	retval = ConfSonars(0, orderlist);
	return retval;
}
